import base64
import json
from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class Ai58(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.Ai58

    # 上传图片, 二进制内容
    def upload(self, content) -> Optional[str]:
        data = {
            'Pic-Size': '0*0',
            'Pic-Encoding': 'base64',
            'Pic-Path': '/nowater/webim/big/',
            'Pic-Data': base64.b64encode(content).decode('utf-8'),
        }
        headers = {
            'Origin': 'https://ai.58.com',
            'Referer': 'https://ai.58.com/pc/'
        }
        response = requests.post('https://upload.58cdn.com.cn/json', data=json.dumps(data), headers=headers)
        if response.status_code != 200:
            return self._set_error('upload fail')
        if not response.text:
            return self._set_error('content is empty')
        return f'https://pic3.58cdn.com.cn/nowater/webim/big/{response.text}'



"""
@author hujing
"""
from enum import Enum, auto


class DepotType(Enum):
    CatBox = auto()
    """
    图床
    url: https://catbox.moe/
    """
    PngCm = auto()
    """
    图床
    url: https://png.cm/
    """
    Riyugo = auto()
    """
    链接存在有效期
    url: https://riyugo.com/
    """
    Ai58 = auto()
    """
    58同城客服, 在线上传图片
    url: https://ai.58.com
    """



"""
定义仓库
@author hujing
"""
import traceback


class DepotError:
    def __init__(self, err, back_num=0):
        """
        :param err:
        :param back_num: 堆栈回溯的数量. 在打印堆栈信息的时候去掉最后几个
        """
        self._err = err
        #  最后一个元素是当前构造函数, 去掉
        self._traceback = traceback.extract_stack()[0:-1-back_num]

    def error(self):
        return self._err

    def traceback(self):
        return self._traceback

    def __str__(self):
        return self._err + '\n' + \
               ''.join(traceback.format_list(self._traceback))



import os
from .base import Depot
from .error import DepotError

# 整理所有的类型 map, 方便获取的时候加速
DEPOT_MAP = {}

"""
将当前目录下的所有文件导入
目的是在使用的时候无需关注具体有哪些子类
添加子类文件后即可创建新的仓库, 无需修改其他额外内容
"""
for file in os.listdir(os.path.dirname(__file__)):
    # 解析, 将文件名后缀去掉
    name = os.path.basename(file).split('.')[0]
    exec(f'from .{name} import *')

for item in Depot.__subclasses__():
    DEPOT_MAP[item.depot_type()] = item




import random
from typing import Optional

import requests

from image_depot import DepotType
from .base import Depot


class PngCm(Depot):
    @classmethod
    def depot_type(cls) -> DepotType:
        return DepotType.PngCm

    def upload(self, content) -> Optional[str]:
        data = {
            'uuid': 'o_1g840' + ''.join(random.sample(list('0123456789abcdefghijklmnopqrstuvwxyz') * 21, 21)),
        }
        files = {
            'file': content, 
        }
        response = requests.post('https://png.cm/application/upload.php', data=data, files=files)
        if response.status_code != 200:
            return self._set_error('upload fail')
        data = response.json()
        if data.get('code') != 200 or not data.get('url'):
            return self._set_error(response.text)
        return data.get('url')



import os
from abc import abstractmethod, ABC
from typing import Optional

import magic

from image_depot import DepotType
from .error import DepotError


class Depot(ABC):
    def __init__(self):
        self._err = None

    @classmethod
    @abstractmethod
    def depot_type(cls) -> DepotType:
        """
        当前类型
        :return:
        """
        pass

    # 上传图片, 二进制内容
    @abstractmethod
    def upload(self, content) -> Optional[str]:
        pass

    # 上传图片, 本地路径
    def upload_file(self, file_path) -> Optional[str]:
        if not os.path.exists(file_path):
            return self._set_error(f'file {file_path} not exist')
        with open(file_path, 'rb') as f:
            return self.upload(f.read())

    def error(self) -> Optional[DepotError]:
        """
        获取错误信息
        :return:
        """
        if self._err:
            return self._err
        return None

    def _set_error(self, err) -> None:
        self._err = DepotError(err, back_num=1)
        return None

    @staticmethod
    def _get_mime_type(content):
        """
        识别文件的 mime type
        :param content:
        :return:
        """
        return magic.Magic(mime=True, uncompress=True).from_buffer(content)
